==> financial-front-2.0.0.tar.gz: financial-front-2.0.0/app/models/user.rb <==
# -*- coding: utf-8 -*-

require 'digest/sha1'

# ユーザ (社員)
class User < ActiveRecord::Base
  has_many :users_divisions
  has_many :divisions, :through => :users_divisions
  
  # Virtual attribute for the unencrypted password
  attr_accessor :password
  attr_accessor :password_confirmation

  validates_presence_of :worker_code
  validates_uniqueness_of :worker_code, :message => "が重複しています。"
  validates_presence_of :name
  validates_presence_of :kana_name
  
  before_save :encrypt_password
  before_save :normalize_string
  
  def normalize_string
    self.name = normalize_kc(name)
    self.kana_name = normalize_kc(kana_name)
    
    self.login = nil if login.to_s.strip == ""
    self.email = nil if email.to_s.strip == ""
  end

  # @override
  def validate
    errors.add "worker_code", "は0以外でなければなりません。" if worker_code == 0
    if !email.blank?
      errors.add 'email', "には半角英数のみ使えます: #{email}" if email =~ /[^a-zA-Z0-9_@.-]/
    end

    if !login.blank?
      errors.add 'email', 'がありません。' if !email || email.strip == ''
      errors.add 'login', 'には半角英数のみ使えます。' if login =~ /[^a-zA-Z0-9_.-]/
      if !password.blank?
        errors.add 'password', 'が短すぎます。' if password.length < 3
        errors.add 'password_confirmation', 'が一致していません。' if password != password_confirmation
      end
    end
  end
  
  # @override
  def validate_on_create
    if !login.blank?
      errors.add 'login', 'が重複しています。' if User.find_by_login(login)
      errors.add 'email', 'が重複しています。' if User.find_by_email(email)
      errors.add 'password', 'がありません。' if password.blank?
    end
  end

  # @override
  def validate_on_update
    if !login.blank?
      if User.find(:first, :conditions => ['login = ? AND id <> ?', login, id])
        errors.add 'login', 'が重複しています。'
      end
      if User.find(:first, :conditions => ['email = ? AND id <> ?', email, id])
        errors.add 'email', 'が重複しています。' 
      end
    end
  end

  
  def has_role?(name)
    if name == :keiri
      return keiri_flag ? true : false
    elsif name == :marketing
      return marketing_flag ? true : false
    elsif name == :hrd
      return hrd_flag ? true : false
    elsif name == :apar
      return apar_flag ? true : false
    else
      raise ArgumentError
    end
  end


  # 主な所属部門
  def default_division
    ud = UsersDivision.find(:first,
                :conditions => ['user_id = ? AND end_day IS NULL AND default_pair <> 0', id])
    ud ? ud.division : nil
  end


  # TODO: 経理承認と全社閲覧を分割すること
  def viewable_cost_centers
    if has_role?(:keiri)
      @cost_centers = ControlDivision.find :all, 
                                      :conditions => ["parent_id IS NOT NULL"]
      @cost_centers.sort! do |x, y| x.root.id <=> y.root.id end
    else
      # ユーザの管理対象部門を含むコストセンタのみ
      cost_centers = ControlDivision.find :all,
             :select => 'DISTINCT control_divisions.*',
             :joins => "JOIN controllers_divisions ON control_divisions.id = controllers_divisions.control_division_id",
             :conditions => ["controllers_divisions.division_id IN (SELECT division_id FROM users_divisions WHERE user_id = ? AND end_day IS NULL) AND parent_id IS NOT NULL", self[:id] ]
      
      # デフォルト集計木のみ表示
      default_root_id = GlobalSetting.first.default_division_tree_root_id
      @cost_centers = cost_centers.delete_if {|r|
        r.root.id != default_root_id
      }
    end
  end


  ###########################################################################
  # from acts_as_authenticated

  # Authenticates a user by their login name and unencrypted password.
  # Returns the user or nil.
  def self.authenticate(login, password)
    u = find :first, :conditions => ['login = ? AND active <> 0', login] # need to get the salt
    # raise "login = '#{login}', #{u.inspect}" DEBUG
    u && u.authenticated?(password) ? u : nil
  end

  # Encrypts some data with the salt.
  def self.encrypt(password, salt)
    Digest::SHA1.hexdigest("--#{salt}--#{password}--")
  end

  # Encrypts the password with the user salt
  def encrypt(password)
    User.encrypt(password, salt)
  end

  def authenticated?(password)
    crypted_password == encrypt(password)
  end

  def remember_token?
    remember_token_expires_at && Time.now.utc < remember_token_expires_at 
  end

  # These create and unset the fields required for remembering users between browser closes
  def remember_me
    self.remember_token_expires_at = 2.weeks.from_now.utc
    self.remember_token            = encrypt("#{email}--#{remember_token_expires_at}")
    save(false)
  end

  def forget_me
    self.remember_token_expires_at = nil
    self.remember_token            = nil
    save(false)
  end

  protected
  # before_save
  def encrypt_password
    return if password.blank?
    self.salt = Digest::SHA1.hexdigest("--#{Time.now.to_s}--#{login}--") if new_record?
    self.crypted_password = encrypt(password)
  end

  def password_required?
    crypted_password.blank? || !password.blank?
  end
  
  # from acts_as_authenticated
  ###########################################################################
end

==> financial-front-2.0.1.tar.gz: financial-front-2.0.1/app/models/user.rb <==
# -*- coding: utf-8 -*-

require 'digest/sha1'

# ユーザ (社員)
class User < ActiveRecord::Base
  has_many :users_divisions
  has_many :divisions, :through => :users_divisions
  
  # Virtual attribute for the unencrypted password
  attr_accessor :password
  attr_accessor :password_confirmation

  validates_presence_of :worker_code
  validates_uniqueness_of :worker_code, :message => "が重複しています。"
  validates_presence_of :name
  validates_presence_of :kana_name
  
  before_save :encrypt_password
  before_save :normalize_string
  
  def normalize_string
    self.name = normalize_kc name
    self.kana_name = normalize_kc kana_name

    self.login = nil if login.to_s.strip == ""
    self.email = nil if email.to_s.strip == ""
  end

  # @override
  def validate
    errors.add "worker_code", "は0以外でなければなりません。" if worker_code == 0
    if !email.blank?
      errors.add 'email', "には半角英数のみ使えます: #{email}" if email =~ /[^a-zA-Z0-9_@.-]/
    end

    if !login.blank?
      errors.add 'email', 'がありません。' if !email || email.strip == ''
      errors.add 'login', 'には半角英数のみ使えます。' if login =~ /[^a-zA-Z0-9_.-]/
      if !password.blank?
        errors.add 'password', 'が短すぎます。' if password.length < 3
        errors.add 'password_confirmation', 'が一致していません。' if password != password_confirmation
      end
    end
  end
  
  # @override
  def validate_on_create
    if !login.blank?
      errors.add 'login', 'が重複しています。' if User.find_by_login(login)
      errors.add 'email', 'が重複しています。' if User.find_by_email(email)
      errors.add 'password', 'がありません。' if password.blank?
    end
  end

  # @override
  def validate_on_update
    if !login.blank?
      if User.find(:first, :conditions => ['login = ? AND id <> ?', login, id])
        errors.add 'login', 'が重複しています。'
      end
      if User.find(:first, :conditions => ['email = ? AND id <> ?', email, id])
        errors.add 'email', 'が重複しています。' 
      end
    end
  end

  
  def has_role?(name)
    if name == :keiri
      return keiri_flag ? true : false
    elsif name == :marketing
      return marketing_flag ? true : false
    elsif name == :hrd
      return hrd_flag ? true : false
    elsif name == :apar
      return apar_flag ? true : false
    else
      raise ArgumentError
    end
  end


  # 主な所属部門
  def default_division
    ud = UsersDivision.find(:first,
                :conditions => ['user_id = ? AND end_day IS NULL AND default_pair <> 0', id])
    ud ? ud.division : nil
  end


  # TODO: 経理承認と全社閲覧を分割すること
  def viewable_cost_centers
    if has_role?(:keiri)
      @cost_centers = ControlDivision.find :all, 
                                      :conditions => ["parent_id IS NOT NULL"]
      @cost_centers.sort! do |x, y| x.root.id <=> y.root.id end
    else
      # ユーザの管理対象部門を含むコストセンタのみ
      cost_centers = ControlDivision.find :all,
             :select => 'DISTINCT control_divisions.*',
             :joins => "JOIN controllers_divisions ON control_divisions.id = controllers_divisions.control_division_id",
             :conditions => ["controllers_divisions.division_id IN (SELECT division_id FROM users_divisions WHERE user_id = ? AND end_day IS NULL) AND parent_id IS NOT NULL", self[:id] ]
      
      # デフォルト集計木のみ表示
      default_root_id = GlobalSetting.first.default_division_tree_root_id
      @cost_centers = cost_centers.delete_if {|r|
        r.root.id != default_root_id
      }
    end
  end


  ###########################################################################
  # from acts_as_authenticated

  # Authenticates a user by their login name and unencrypted password.
  # Returns the user or nil.
  def self.authenticate(login, password)
    u = find :first, :conditions => ['login = ? AND active <> 0', login] # need to get the salt
    # raise "login = '#{login}', #{u.inspect}" DEBUG
    u && u.authenticated?(password) ? u : nil
  end

  # Encrypts some data with the salt.
  def self.encrypt(password, salt)
    Digest::SHA1.hexdigest("--#{salt}--#{password}--")
  end

  # Encrypts the password with the user salt
  def encrypt(password)
    User.encrypt(password, salt)
  end

  def authenticated?(password)
    crypted_password == encrypt(password)
  end

  def remember_token?
    remember_token_expires_at && Time.now.utc < remember_token_expires_at 
  end

  # These create and unset the fields required for remembering users between browser closes
  def remember_me
    self.remember_token_expires_at = 2.weeks.from_now.utc
    self.remember_token            = encrypt("#{email}--#{remember_token_expires_at}")
    save(false)
  end

  def forget_me
    self.remember_token_expires_at = nil
    self.remember_token            = nil
    save(false)
  end

  protected
  # before_save
  def encrypt_password
    return if password.blank?
    self.salt = Digest::SHA1.hexdigest("--#{Time.now.to_s}--#{login}--") if new_record?
    self.crypted_password = encrypt(password)
  end

  def password_required?
    crypted_password.blank? || !password.blank?
  end
  
  # from acts_as_authenticated
  ###########################################################################
end
